package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.ConceptDetailDTO;
import com.diagbot.dto.ConceptIntroduceDTO;
import com.diagbot.entity.Concept;
import com.diagbot.entity.ConceptDetail;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.ConceptDetailServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.ParamConvertUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.ConceptBaseVO;
import com.diagbot.vo.ConceptIntroduceVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2019/5/6 15:03
 */
@Component
public class ConceptDetailFacade extends ConceptDetailServiceImpl {
    @Autowired
    private ConceptFacade conceptFacade;

    /**
     * 获取提示信息
     *
     * @param conceptIntroduceVO
     * @return
     */
    public ConceptIntroduceDTO getConceptDetail(ConceptIntroduceVO conceptIntroduceVO) {
        if (StringUtil.isBlank(conceptIntroduceVO.getName())) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "请输入概念名称");
        }
        conceptIntroduceVO.setLibType(ParamConvertUtil.conceptConvert2Lib(conceptIntroduceVO.getLibType()));
        //验证存在并且获取概念基本信息
        Concept concept = conceptFacade.exist(conceptIntroduceVO);

        //获取概念提示信息
        QueryWrapper<ConceptDetail> conceptDetailQueryWrapper = new QueryWrapper<>();
        conceptDetailQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("concept_id", concept.getId())
                .apply("find_in_set({0},position)", conceptIntroduceVO.getPosition())
                .orderByAsc("order_no");
        List<ConceptDetail> conceptDetailList = this.list(conceptDetailQueryWrapper);
        List<ConceptDetailDTO> conceptDetailDTOList = ListUtil.newArrayList();
        if (ListUtil.isNotEmpty(conceptDetailList)) {
            conceptDetailDTOList = BeanUtil.listCopyTo(conceptDetailList, ConceptDetailDTO.class);
        }
        ConceptIntroduceDTO conceptIntroduceDTO = new ConceptIntroduceDTO();
        conceptIntroduceDTO.setConceptId(concept.getId());
        conceptIntroduceDTO.setName(concept.getLibName());
        conceptIntroduceDTO.setDetails(conceptDetailDTOList);
        return conceptIntroduceDTO;
    }

    /**
     * 获取静态知识明细
     *
     * @param conceptIntroduceVO
     * @return
     */
    public List<ConceptDetail> getConceptDetailList(ConceptIntroduceVO conceptIntroduceVO) {
        //验证存在并且获取概念基本信息
        Concept concept = conceptFacade.exist(conceptIntroduceVO);

        //获取概念提示信息
        QueryWrapper<ConceptDetail> conceptDetailQueryWrapper = new QueryWrapper<>();
        conceptDetailQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("concept_id", concept.getId())
                .apply("find_in_set({0},position)", conceptIntroduceVO.getPosition())
                .orderByAsc("order_no");
        List<ConceptDetail> conceptDetailList = this.list(conceptDetailQueryWrapper);
        return conceptDetailList;
    }

    /**
     * 是否存在提示信息
     *
     * @param conceptBaseVO
     * @return
     */
    public Boolean exist(ConceptBaseVO conceptBaseVO) {
        //验证存在并且获取概念基本信息
        Concept concept = conceptFacade.exist(conceptBaseVO);
        QueryWrapper<ConceptDetail> conceptDetailQueryWrapper = new QueryWrapper<>();
        conceptDetailQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("concept_id", concept.getId());
        List<ConceptDetail> conceptDetailList = this.list(conceptDetailQueryWrapper);
        if (ListUtil.isNotEmpty(conceptDetailList)) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 是否存在提示信息-列表
     *
     * @param nameList
     * @param libType
     * @return
     */
    public Map<String, Concept> existList(List<String> nameList, Integer libType) {
        List<Concept> concepts = conceptFacade.getListByNamesAndType(nameList, libType);
        Map<String, Concept> retMap = new LinkedHashMap<>();
        if (ListUtil.isNotEmpty(concepts)) {
            List<Long> conceptIds = concepts.stream().map(concept -> concept.getId()).collect(Collectors.toList());
            QueryWrapper<ConceptDetail> conceptDetailQueryWrapper = new QueryWrapper<>();
            conceptDetailQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                    .in("concept_id", conceptIds);
            List<ConceptDetail> conceptDetailList = this.list(conceptDetailQueryWrapper);
            if (ListUtil.isNotEmpty(conceptDetailList)) {
                Map<Long, ConceptDetail> conceptDetailMap = EntityUtil.makeEntityMap(conceptDetailList, "conceptId");
                for (Concept concept : concepts) {
                    if (conceptDetailMap.containsKey(concept.getId()) && conceptDetailMap.get(concept.getId()) != null) {
                        retMap.put(concept.getLibName(), concept);
                    }
                }
            }
        }
        return retMap;
    }
}